﻿//----------------------------------------------------------------------------------------
// name: Chicken Sequencer (Chickencer)
// desc: demonstration of tight timing control from chuck, with communication with unity
//
// author: Ge Wang (https://www.gewang.com/)
// date: Fall 2019
//----------------------------------------------------------------------------------------
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ChickenSequencer : MonoBehaviour
{
    // get the chicken factory
    public ChickenFactory m_chickenFactory;
    // float sync
    private ChuckFloatSyncer m_ckPlayPos;
    // int sync
    private ChuckIntSyncer m_ckCurrent;
    // get the chickens
    public GameObject[] m_chickens;
    // the playhead object
    public GameObject m_playHead;
    // previous step
    private int m_previous = -1;
    // which selected
    private int m_selected = 0;

    // sequence: this information is duplicated across Chuck and Unity
    // and is communicated to chuck using ChucK events
    private float[] m_seqRate = new float[] { 1f, 1f, 1f, 1f};
    private float[] m_seqGain = new float[] { 1f, 1f, 1f, 1f };

    // Start is called before the first frame update
    void Start()
    {
        // find the chicken factory
        GameObject go = GameObject.Find("TheChickenFactory");
        // get the chicken factory instance
        m_chickenFactory = go.GetComponent<ChickenFactory>();

        // run the sequencer
        GetComponent<ChuckSubInstance>().RunFile("chickencer.ck", true);

        // add the float sync
        m_ckPlayPos = gameObject.AddComponent<ChuckFloatSyncer>();
        m_ckPlayPos.SyncFloat(GetComponent<ChuckSubInstance>(), "playPos");
        // add the int sync
        m_ckCurrent = gameObject.AddComponent<ChuckIntSyncer>();
        m_ckCurrent.SyncInt(GetComponent<ChuckSubInstance>(), "current");
    }

    // Update is called once per frame
    void Update()
    {
        // get input
        if (Input.GetKeyDown("1"))
        {
            m_selected = 0;
        }
        else if (Input.GetKeyDown("2"))
        {
            m_selected = 1;
        }
        else if (Input.GetKeyDown("3"))
        {
            m_selected = 2;
        }
        else if (Input.GetKeyDown("4"))
        {
            m_selected = 3;
        }
        else if (Input.GetKeyDown("left"))
        {
            AdjustRate(m_selected, true);
        }
        else if (Input.GetKeyDown("right"))
        {
            AdjustRate(m_selected, false);
        }
        else if (Input.GetKeyDown("up"))
        {
            AdjustGain(m_selected, true);
        }
        else if (Input.GetKeyDown("down"))
        {
            AdjustGain(m_selected, false);
        }

        // offset
        float offset = m_chickenFactory.the_chickenSpacing * (m_chickenFactory.the_numChickens - 1);
        // position
        m_playHead.transform.position = new Vector3(-offset / 2 + m_chickenFactory.the_chickenSpacing * m_ckPlayPos.GetCurrentValue(), 0, -.15f);
        // get current step number
        int current = m_ckCurrent.GetCurrentValue();
        // should eat?
        if (current != m_previous)
        {
            // chicken, eat!
            Eat(current);
            // remember
            m_previous = current;
        }
    }

    // Eat
    void Eat( int which )
    {
        // get the chickens
        m_chickens = m_chickenFactory.the_chickens;
        // eat!
        m_chickens[which].GetComponent<Animator>().Play("Eat", -1, 0f);
    }

    // AdjustRate
    void AdjustRate( int which, bool isLeft )
    {
        // ajdust
        if( !isLeft )
        {
            m_seqRate[which] *= .9f;
            if (m_seqRate[which] < .5f) m_seqRate[which] = .5f;
        }
        else
        {
            m_seqRate[which] *= 1.1f;
            if (m_seqRate[which] > 2.0f) m_seqRate[which] = 2.0f;
        }

        // get the chickens
        m_chickens = m_chickenFactory.the_chickens;
        // eat!
        m_chickens[which].transform.localScale = new Vector3(1f/m_seqRate[which], m_seqGain[which], 1);


        // set which and rate and fire the event
        GetComponent<ChuckSubInstance>().SetInt("editWhich", which);
        GetComponent<ChuckSubInstance>().SetFloat("editRate", m_seqRate[which]);
        GetComponent<ChuckSubInstance>().SetFloat("editGain", m_seqGain[which]);
        GetComponent<ChuckSubInstance>().BroadcastEvent("editHappened");
    }

    // AdjustRate
    void AdjustGain(int which, bool isUp)
    {
        // ajdust
        if (isUp)
        {
            m_seqGain[which] *= 1.1f;
            if (m_seqGain[which] > 5f) m_seqGain[which] = 5f;
        }
        else
        {
            m_seqGain[which] *= .9f;
            if (m_seqGain[which] < .2f) m_seqGain[which] = .2f;
        }

        // get the chickens
        m_chickens = m_chickenFactory.the_chickens;
        // eat!
        m_chickens[which].transform.localScale = new Vector3(1f / m_seqRate[which], m_seqGain[which], 1);

        // set which and rate and fire the event
        GetComponent<ChuckSubInstance>().SetInt("editWhich", which);
        GetComponent<ChuckSubInstance>().SetFloat("editRate", m_seqRate[which]);
        GetComponent<ChuckSubInstance>().SetFloat("editGain", m_seqGain[which]);
        GetComponent<ChuckSubInstance>().BroadcastEvent("editHappened");
    }
}
