//---------------------------------------------
// name: chickencer.ck
// desc: a Chicken Sequencer (Chunity Demo)
//
// author: Ge Wang (ge@ccrma.stanford.edu)
// date: Fall 2019
//---------------------------------------------

// number of chickens
4 => int NUM_CHICKENS;
// beat (smallest division; dur between chickens)
400::ms => dur BEAT;
// update rate for pos
10::ms => dur POS_RATE;
// increment per setp
POS_RATE/BEAT => float posInc;

// global (outgoing) variables for Unity
global int current; // which chicken we are at
global float playPos; // will take on values [0-NUM_CHICKENS)

// global (incoming) data from Unity
global int editWhich;
1 => global float editRate;
1 => global float editGain;
global Event editHappened;

// a sequence (one of many ways to "sequence")
// this ones treats chickens as a step sequencer
float seqRate[NUM_CHICKENS];
float seqGain[NUM_CHICKENS];
// initialize
for( int i; i < NUM_CHICKENS; i++ )
{
    1 => seqGain[i];
    1 => seqRate[i];
}
    
// sound buffers
SndBuf bufs[NUM_CHICKENS];
// reverb
NRev reverb => dac;
// reverb mix
.1 => reverb.mix;

// connect them
for( int i; i < NUM_CHICKENS; i++ )
{
    // connect to dac
    bufs[i] => reverb;
    // load sound
    "special:dope" => bufs[i].read;
    // silence
    0 => bufs[i].gain;
}

// this updates the global playPos with fine granularity,
// for visualizing the playhead smoothly in Unity
fun void playPosUpdate()
{
    while( true )
    {
        // increment
        posInc +=> playPos;
        // advance time
        POS_RATE => now;
    }
}

// this listens for events from Unity to update the sequence
fun void listenForEdit()
{
    while( true )
    {
        // wait for event
        editHappened => now;
        // update the gain and rate
        editRate => seqRate[editWhich];
        editGain => seqGain[editWhich];
        
        <<< "EDIT rate:", editRate, "gain:", editGain >>>;
    }
}

// spork update
spork ~ playPosUpdate();
// spork edit listener
spork ~ listenForEdit();

// main time loop
while( true )
{
    // play current
    play( current, seqGain[current], seqRate[current] );
    // sync with pos
    current => playPos;
    // go with the beat
    BEAT => now;
    // increment
    current++;
    // wrap
    if( current >= NUM_CHICKENS ) 0 => current;
}

fun void play( int which, float gain, float rate )
{
    // restart
    0 => bufs[which].pos;
    // set gain
    gain => bufs[which].gain;
    // set rate
    rate => bufs[which].rate;
}
